@@ -16,7 +16,6 @@
 #include <midori/midori.h>
 
 #define EXTENSION_NAME "Feed Panel"
-#define UPDATE_FREQ 10
 
 #define feed_get_flags(feed) \
     GPOINTER_TO_INT (g_object_get_data (G_OBJECT ((feed)), "flags"))
@@ -102,14 +101,6 @@ feed_deactivate_cb (MidoriExtension* extension,
     }
 }
 
-static void
-feed_dialog_response_cb (GtkWidget* dialog,
-                         gint       response,
-                         gpointer   data)
-{
-    gtk_widget_destroy (dialog);
-}
-
 static KatzeArray*
 feed_add_item (KatzeArray*  feeds,
                const gchar* uri)
@@ -131,8 +122,8 @@ feed_add_item (KatzeArray*  feeds,
                 _("Feed '%s' already exists"), uri);
             gtk_window_set_title (GTK_WINDOW (dialog), EXTENSION_NAME);
             gtk_widget_show (dialog);
-            g_signal_connect (dialog, "response",
-                    G_CALLBACK (feed_dialog_response_cb), NULL);
+            g_signal_connect_swapped (dialog, "response",
+                G_CALLBACK (gtk_widget_destroy), dialog);
 
         }
         else
@@ -488,8 +479,8 @@ feed_app_add_browser_cb (MidoriApp*       app,
     g_signal_connect (extension, "deactivate",
         G_CALLBACK (feed_deactivate_cb), priv);
 
-    priv->source_id = g_timeout_add_seconds (UPDATE_FREQ * 60,
-                            (GSourceFunc) update_feeds, priv);
+    priv->source_id = midori_timeout_add_seconds (
+        600, (GSourceFunc) update_feeds, priv, NULL);
 }
 
 static void
