@@ -14,9 +14,7 @@
 FormHistoryPriv*
 formhistory_private_new ()
 {
-    FormHistoryPriv* priv;
-
-    priv = g_slice_new (FormHistoryPriv);
+    FormHistoryPriv* priv = g_slice_new (FormHistoryPriv);
     priv->oldkeyword = g_strdup ("");
     priv->selection_index = -1;
     return priv;
@@ -367,8 +365,8 @@ formhistory_editbox_key_pressed_cb (WebKitDOMElement* element,
     if (gtk_widget_get_visible (priv->popup) &&
         !g_strcmp0 (keyword, priv->oldkeyword))
         goto free_data;
-    priv->completion_timeout = g_timeout_add (COMPLETION_DELAY,
-                               (GSourceFunc)formhistory_suggestions_show, priv);
+    priv->completion_timeout = midori_timeout_add (COMPLETION_DELAY,
+        (GSourceFunc)formhistory_suggestions_show, priv, NULL);
 free_data:
     g_free (keyword);
 }
