@@ -46,8 +46,6 @@ static const GtkTargetEntry tb_editor_dnd_targets[] =
 };
 static const gint tb_editor_dnd_targets_len = G_N_ELEMENTS(tb_editor_dnd_targets);
 
-static void tb_editor_browser_populate_tool_menu_cb(MidoriBrowser *browser, GtkWidget *menu, MidoriExtension *ext);
-
 static void tb_editor_browser_populate_toolbar_menu_cb(MidoriBrowser *browser, GtkWidget *menu,
                                                        MidoriExtension *ext);
 
@@ -58,7 +56,6 @@ static void tb_editor_deactivate_cb(MidoriExtension *extension, MidoriBrowser *b
 {
 	MidoriApp *app = midori_extension_get_app(extension);
 
-	g_signal_handlers_disconnect_by_func(browser, tb_editor_browser_populate_tool_menu_cb, extension);
 	g_signal_handlers_disconnect_by_func(browser, tb_editor_browser_populate_toolbar_menu_cb, extension);
 	g_signal_handlers_disconnect_by_func(extension, tb_editor_deactivate_cb, browser);
 	g_signal_handlers_disconnect_by_func(app, tb_editor_app_add_browser_cb, extension);
@@ -107,7 +104,12 @@ static GSList *tb_editor_array_to_list(const gchar **items)
 	name = items;
 	while (*name != NULL)
 	{
+		#ifdef HAVE_GRANITE
+		/* A "new tab" button is already part of the notebook */
+		if (*name[0] != '\0' && strcmp (*name, "TabNew"))
+		#else
 		if (*name[0] != '\0')
+		#endif
 			list = g_slist_append(list, g_strdup(*name));
 		name++;
 	}
@@ -274,16 +276,15 @@ static void tb_editor_drag_data_rcvd_cb(GtkWidget *widget, GdkDragContext *conte
 										gint x, gint y, GtkSelectionData *data, guint info,
 										guint ltime, TBEditorWidget *tbw)
 {
-#if !GTK_CHECK_VERSION(3,0,0) /* TODO */
 	GtkTreeView *tree = GTK_TREE_VIEW(widget);
 	gboolean del = FALSE;
 
-	if (data->length >= 0 && data->format == 8)
+	if (gtk_selection_data_get_length (data) >= 0 && gtk_selection_data_get_format (data) == 8)
 	{
 		gboolean is_sep;
 		gchar *text = NULL;
 
-		text = (gchar*) data->data;
+		text = (gchar*) gtk_selection_data_get_data (data);
 
 		/* We allow re-ordering the Location item but not removing it from the list. */
 		if (g_strcmp0(text, "Location") == 0 && widget != tbw->drag_source)
@@ -332,7 +333,6 @@ static void tb_editor_drag_data_rcvd_cb(GtkWidget *widget, GdkDragContext *conte
 	tbw->drag_source = NULL; /* reset the value just to be sure */
 	tb_editor_free_path(tbw);
 	gtk_drag_finish(context, TRUE, del, ltime);
-#endif
 }
 
 
@@ -574,17 +574,6 @@ static void tb_editor_menu_configure_toolbar_activate_cb(GtkWidget *menuitem, Mi
 	g_free(tbw);
 }
 
-static void tb_editor_browser_populate_tool_menu_cb(MidoriBrowser *browser, GtkWidget *menu, MidoriExtension *ext)
-{
-    GtkWidget *menuitem;
-
-    menuitem = gtk_menu_item_new_with_mnemonic (_("Customize _Toolbar..."));
-    g_signal_connect (menuitem, "activate",
-        G_CALLBACK (tb_editor_menu_configure_toolbar_activate_cb), browser);
-    gtk_widget_show (menuitem);
-    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-}
-
 static void tb_editor_browser_populate_toolbar_menu_cb(MidoriBrowser *browser, GtkWidget *menu,
                                                        MidoriExtension *ext)
 {
@@ -594,7 +583,7 @@ static void tb_editor_browser_populate_toolbar_menu_cb(MidoriBrowser *browser, G
     separator = gtk_separator_menu_item_new ();
     gtk_widget_show (separator);
     gtk_menu_shell_append (GTK_MENU_SHELL (menu), separator);
-    menuitem = gtk_menu_item_new_with_mnemonic (_("_Customize..."));
+    menuitem = gtk_menu_item_new_with_mnemonic (_("_Customize Toolbar…"));
     g_signal_connect (menuitem, "activate",
         G_CALLBACK (tb_editor_menu_configure_toolbar_activate_cb), browser);
     gtk_widget_show (menuitem);
@@ -603,7 +592,6 @@ static void tb_editor_browser_populate_toolbar_menu_cb(MidoriBrowser *browser, G
 
 static void tb_editor_app_add_browser_cb(MidoriApp *app, MidoriBrowser *browser, MidoriExtension *ext)
 {
-    g_signal_connect(browser, "populate-tool-menu", G_CALLBACK(tb_editor_browser_populate_tool_menu_cb), ext);
     g_signal_connect(browser, "populate-toolbar-menu", G_CALLBACK(tb_editor_browser_populate_toolbar_menu_cb), ext);
     g_signal_connect(ext, "deactivate", G_CALLBACK(tb_editor_deactivate_cb), browser);
 }
