@@ -447,7 +447,7 @@ web_cache_activate_cb (MidoriExtension* extension,
 static void
 web_cache_clear_cache_cb (void)
 {
-    sokoke_remove_path (web_cache_get_cache_dir (), TRUE);
+    midori_paths_remove_path (web_cache_get_cache_dir ());
 }
 #endif
 
@@ -467,7 +467,7 @@ extension_init (void)
     g_signal_connect (extension, "activate",
         G_CALLBACK (web_cache_activate_cb), NULL);
 
-    sokoke_register_privacy_item ("web-cache", _("Web Cache"),
+    midori_private_data_register_item ("web-cache", _("Web Cache"),
         G_CALLBACK (web_cache_clear_cache_cb));
 
     return extension;
