@@ -70,6 +70,24 @@ proxy_uri_file_set_cb (GtkFileChooser* button,
     g_object_set (object, property, file, NULL);
 }
 
+#if GTK_CHECK_VERSION (3, 2, 0)
+static void
+proxy_font_chooser_font_activated_cb (GtkFontChooser* chooser,
+                                      const gchar*    font_name,
+                                      GObject*        object)
+{
+    gtk_font_chooser_set_font (chooser, font_name);
+}
+
+static gboolean
+proxy_font_chooser_filter_monospace_cb (PangoFontFamily* family,
+                                        PangoFontFace*   face,
+                                        gpointer         data)
+{
+    gboolean monospace = GPOINTER_TO_INT (data);
+    return monospace == pango_font_family_is_monospace (family);
+}
+#else
 static void
 proxy_combo_box_text_changed_cb (GtkComboBoxText* button,
                                  GObject*         object)
@@ -79,6 +97,7 @@ proxy_combo_box_text_changed_cb (GtkComboBoxText* button,
     g_object_set (object, property, text, NULL);
     g_free (text);
 }
+#endif
 
 static const gchar*
 katze_app_info_get_commandline (GAppInfo* info)
@@ -316,10 +335,11 @@ katze_app_info_get_all_for_category (const gchar* category)
     GList* all_apps = g_app_info_get_all ();
     #endif
     GList* apps = NULL;
-    guint i = 0;
     GAppInfo* info;
-    while ((info = g_list_nth_data (all_apps, i++)))
+    GList* app;
+    for (app = apps; app; app = g_list_next (app))
     {
+        GAppInfo* info = app->data;
         #ifdef GDK_WINDOWING_X11
         gchar* filename = g_strconcat ("applications/", g_app_info_get_id (info), NULL);
         GKeyFile* file = g_key_file_new ();
@@ -343,6 +363,92 @@ katze_app_info_get_all_for_category (const gchar* category)
     return apps;
 }
 
+static gboolean
+proxy_populate_apps (GtkWidget* widget)
+{
+    const gchar* property = g_object_get_data (G_OBJECT (widget), "property");
+    GObject* object = g_object_get_data (G_OBJECT (widget), "object");
+    gchar* string = katze_object_get_string (object, property);
+    if (!g_strcmp0 (string, ""))
+        katze_assign (string, NULL);
+    GtkSettings* settings = gtk_widget_get_settings (widget);
+    gint icon_width = 16;
+    if (settings == NULL)
+        settings = gtk_settings_get_for_screen (gdk_screen_get_default ());
+    gtk_icon_size_lookup_for_settings (settings, GTK_ICON_SIZE_MENU,
+                                       &icon_width, NULL);
+
+    GtkComboBox* combo = GTK_COMBO_BOX (widget);
+    GtkListStore* model = GTK_LIST_STORE (gtk_combo_box_get_model (combo));
+    GtkTreeIter iter_none;
+    gtk_list_store_insert_with_values (model, &iter_none, 0,
+        0, NULL, 1, NULL, 2, _("None"), 3, icon_width, -1);
+
+    const gchar* app_type = g_object_get_data (G_OBJECT (widget), "app-type");
+    GList* apps = g_app_info_get_all_for_type (app_type);
+    GAppInfo* info;
+    if (!apps)
+        apps = katze_app_info_get_all_for_category (app_type);
+    if (apps != NULL)
+    {
+        GList* app;
+        for (app = apps; app; app = g_list_next (app))
+        {
+            GAppInfo* info = app->data;
+            const gchar* name = g_app_info_get_name (info);
+            GIcon* icon = g_app_info_get_icon (info);
+            gchar* icon_name;
+            GtkTreeIter iter;
+
+            if (!g_app_info_should_show (info))
+                continue;
+
+            icon_name = icon ? g_icon_to_string (icon) : NULL;
+            gtk_list_store_insert_with_values (model, &iter, G_MAXINT,
+                0, info, 1, icon_name, 2, name, 3, icon_width, -1);
+            if (string && !strcmp (katze_app_info_get_commandline (info), string))
+                gtk_combo_box_set_active_iter (combo, &iter);
+
+            g_free (icon_name);
+        }
+        g_list_free (apps);
+    }
+
+    info = g_app_info_create_from_commandline ("",
+        "", G_APP_INFO_CREATE_NONE, NULL);
+    gtk_list_store_insert_with_values (model, NULL, G_MAXINT,
+        0, info, 1, NULL, 2, _("Custom…"), 3, icon_width, -1);
+    g_object_unref (info);
+
+    if (gtk_combo_box_get_active (combo) == -1)
+    {
+        if (string)
+        {
+            GtkWidget* entry;
+            const gchar* exe;
+
+            info = g_app_info_create_from_commandline (string,
+                NULL, G_APP_INFO_CREATE_NONE, NULL);
+            entry = gtk_entry_new ();
+            exe = g_app_info_get_executable (info);
+            if (exe && *exe && strcmp (exe, "%f"))
+                gtk_entry_set_text (GTK_ENTRY (entry), string);
+            gtk_widget_show (entry);
+            gtk_container_add (GTK_CONTAINER (combo), entry);
+            g_object_unref (info);
+            g_signal_connect (entry, "focus-out-event",
+                G_CALLBACK (proxy_entry_focus_out_event_cb), object);
+            g_object_set_data_full (G_OBJECT (entry), "property",
+                                    g_strdup (property), g_free);
+        }
+        else
+            gtk_combo_box_set_active_iter (combo, &iter_none);
+    }
+    g_signal_connect (widget, "changed",
+                      G_CALLBACK (proxy_combo_box_apps_changed_cb), object);
+    return G_SOURCE_REMOVE;
+}
+
 /**
  * katze_property_proxy:
  * @object: a #GObject
@@ -497,30 +603,40 @@ katze_property_proxy (gpointer     object,
     else if (type == G_TYPE_PARAM_STRING && (_hint == I_("font")
         || _hint == I_("font-monospace")))
     {
-        GtkComboBox* combo;
-        gint n_families, i;
-        PangoContext* context;
-        PangoFontFamily** families;
-        gboolean monospace = _hint == I_("font-monospace");
         string = katze_object_get_string (object, property);
-
-        widget = gtk_combo_box_text_new ();
-        combo = GTK_COMBO_BOX (widget);
-        context = gtk_widget_get_pango_context (widget);
-        pango_context_list_families (context, &families, &n_families);
         if (!string)
             string = g_strdup (G_PARAM_SPEC_STRING (pspec)->default_value);
         /* 'sans' and 'sans-serif' are presumably the same */
         if (!g_strcmp0 (string, "sans-serif"))
             katze_assign (string, g_strdup ("sans"));
+        gboolean monospace = _hint == I_("font-monospace");
+
+        #if GTK_CHECK_VERSION (3, 2, 0)
+        widget = gtk_font_button_new ();
+        gtk_font_button_set_show_size (GTK_FONT_BUTTON (widget), FALSE);
+        gtk_font_chooser_set_font (GTK_FONT_CHOOSER (widget), string);
+        g_signal_connect (widget, "font-activated",
+                          G_CALLBACK (proxy_font_chooser_font_activated_cb), object);
+        gtk_font_chooser_set_filter_func (GTK_FONT_CHOOSER (widget),
+            (GtkFontFilterFunc)proxy_font_chooser_filter_monospace_cb, GINT_TO_POINTER (monospace), NULL);
+        #else
+        GtkComboBox* combo;
+        gint n_families, i;
+        PangoContext* context;
+        PangoFontFamily** families;
+
+        widget = gtk_combo_box_text_new ();
+        combo = GTK_COMBO_BOX (widget);
+        context = gtk_widget_get_pango_context (widget);
+        pango_context_list_families (context, &families, &n_families);
         if (string)
         {
             gint j = 0;
             for (i = 0; i < n_families; i++)
             {
-                const gchar* font = pango_font_family_get_name (families[i]);
                 if (monospace != pango_font_family_is_monospace (families[i]))
                     continue;
+                const gchar* font = pango_font_family_get_name (families[i]);
                 gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), font);
                 if (!g_ascii_strcasecmp (font, string))
                     gtk_combo_box_set_active (combo, j);
@@ -532,22 +648,13 @@ katze_property_proxy (gpointer     object,
         g_signal_connect (widget, "changed",
                           G_CALLBACK (proxy_combo_box_text_changed_cb), object);
         g_free (families);
+        #endif
     }
     else if (type == G_TYPE_PARAM_STRING && hint && g_str_has_prefix (hint, "application-"))
     {
         GtkListStore* model;
         GtkCellRenderer* renderer;
-        GtkComboBox* combo;
-        GList* apps;
         const gchar* app_type = &hint[12];
-        GtkSettings* settings;
-        gint icon_width = 16;
-        GtkTreeIter iter_none;
-        GAppInfo* info;
-
-        settings = gtk_settings_get_for_screen (gdk_screen_get_default ());
-        gtk_icon_size_lookup_for_settings (settings, GTK_ICON_SIZE_MENU,
-                                           &icon_width, NULL);
 
         model = gtk_list_store_new (4, G_TYPE_APP_INFO, G_TYPE_STRING,
                                        G_TYPE_STRING, G_TYPE_INT);
@@ -559,77 +666,10 @@ katze_property_proxy (gpointer     object,
         renderer = gtk_cell_renderer_text_new ();
         gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (widget), renderer, TRUE);
         gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (widget), renderer, "text", 2);
-        combo = GTK_COMBO_BOX (widget);
-        apps = g_app_info_get_all_for_type (app_type);
-        if (!apps)
-            apps = katze_app_info_get_all_for_category (app_type);
 
-        string = katze_object_get_string (object, property);
-        if (!g_strcmp0 (string, ""))
-            katze_assign (string, NULL);
-
-        gtk_list_store_insert_with_values (model, &iter_none, 0,
-            0, NULL, 1, NULL, 2, _("None"), 3, icon_width, -1);
-
-        if (apps != NULL)
-        {
-            gint i = 0;
-
-            while ((info = g_list_nth_data (apps, i++)))
-            {
-                const gchar* name = g_app_info_get_name (info);
-                GIcon* icon = g_app_info_get_icon (info);
-                gchar* icon_name;
-                GtkTreeIter iter;
-
-                if (!g_app_info_should_show (info))
-                    continue;
-
-                icon_name = icon ? g_icon_to_string (icon) : NULL;
-                gtk_list_store_insert_with_values (model, &iter, G_MAXINT,
-                    0, info, 1, icon_name, 2, name, 3, icon_width, -1);
-                if (string && !strcmp (katze_app_info_get_commandline (info), string))
-                    gtk_combo_box_set_active_iter (combo, &iter);
-
-                g_free (icon_name);
-            }
-            g_list_free (apps);
-        }
-
-        {
-            info = g_app_info_create_from_commandline ("",
-                "", G_APP_INFO_CREATE_NONE, NULL);
-            gtk_list_store_insert_with_values (model, NULL, G_MAXINT,
-                0, info, 1, NULL, 2, _("Custom..."), 3, icon_width, -1);
-            g_object_unref (info);
-
-            if (gtk_combo_box_get_active (combo) == -1)
-            {
-                if (string)
-                {
-                    GtkWidget* entry;
-                    const gchar* exe;
-
-                    info = g_app_info_create_from_commandline (string,
-                        NULL, G_APP_INFO_CREATE_NONE, NULL);
-                    entry = gtk_entry_new ();
-                    exe = g_app_info_get_executable (info);
-                    if (exe && *exe && strcmp (exe, "%f"))
-                        gtk_entry_set_text (GTK_ENTRY (entry), string);
-                    gtk_widget_show (entry);
-                    gtk_container_add (GTK_CONTAINER (combo), entry);
-                    g_object_unref (info);
-                    g_signal_connect (entry, "focus-out-event",
-                        G_CALLBACK (proxy_entry_focus_out_event_cb), object);
-                    g_object_set_data_full (G_OBJECT (entry), "property",
-                                            g_strdup (property), g_free);
-                }
-                else
-                    gtk_combo_box_set_active_iter (combo, &iter_none);
-            }
-        }
-        g_signal_connect (widget, "changed",
-                          G_CALLBACK (proxy_combo_box_apps_changed_cb), object);
+        g_object_set_data_full (G_OBJECT (widget), "app-type", g_strdup (app_type), g_free);
+        g_object_set_data_full (G_OBJECT (widget), "object", g_object_ref (object), g_object_unref);
+        g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc)proxy_populate_apps, widget, NULL);
     }
     else if (type == G_TYPE_PARAM_STRING)
     {
@@ -667,7 +707,8 @@ katze_property_proxy (gpointer     object,
     }
     else if (type == G_TYPE_PARAM_FLOAT)
     {
-        gfloat value = katze_object_get_float (object, property);
+        gfloat value;
+        g_object_get (object, property, &value, NULL);
 
         widget = gtk_spin_button_new_with_range (
             G_PARAM_SPEC_FLOAT (pspec)->minimum,
@@ -781,45 +822,6 @@ katze_property_proxy (gpointer     object,
     return widget;
 }
 
-/**
- * katze_property_label:
- * @object: a #GObject
- * @property: the name of a property
- *
- * Create a label widget displaying the name of the specified object's property.
- *
- * Return value: a new label widget
- *
- * Since 0.2.1 the label will be empty if the property proxy for the
- *    same property would contain a label already.
- **/
-GtkWidget*
-katze_property_label (gpointer     object,
-                      const gchar* property)
-{
-    GObjectClass* class;
-    GParamSpec* pspec;
-    const gchar* nick;
-    GtkWidget* widget;
-
-    g_return_val_if_fail (G_IS_OBJECT (object), NULL);
-
-    class = G_OBJECT_GET_CLASS (object);
-    pspec = g_object_class_find_property (class, property);
-    if (!pspec)
-    {
-        g_warning (_("Property '%s' is invalid for %s"),
-                   property, G_OBJECT_CLASS_NAME (class));
-        return gtk_label_new (property);
-    }
-
-    nick = g_param_spec_get_nick (pspec);
-    widget = gtk_label_new (nick);
-    gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
-
-    return widget;
-}
-
 typedef struct
 {
      GtkWidget* widget;
@@ -954,59 +956,6 @@ katze_image_menu_item_new_ellipsized (const gchar* label)
     return menuitem;
 }
 
-/**
- * katze_pixbuf_new_from_buffer:
- * @buffer: Buffer with image data
- * @length: Length of the buffer
- * @mime_type: a MIME type, or %NULL
- * @error: return location for a #GError, or %NULL
- *
- * Creates a new #GdkPixbuf out of the specified buffer.
- *
- * You can specify a MIME type if looking at the buffer
- * is not enough to determine the right type.
- *
- * Return value: A newly-allocated #GdkPixbuf
- **/
-GdkPixbuf*
-katze_pixbuf_new_from_buffer (const guchar* buffer,
-                              gsize         length,
-                              const gchar*  mime_type,
-                              GError**      error)
-{
-    /* Proposed for inclusion in GdkPixbuf
-       See http://bugzilla.gnome.org/show_bug.cgi?id=74291 */
-    GdkPixbufLoader* loader;
-    GdkPixbuf* pixbuf;
-
-    g_return_val_if_fail (buffer != NULL, NULL);
-    g_return_val_if_fail (length > 0, NULL);
-
-    if (mime_type)
-    {
-        loader = gdk_pixbuf_loader_new_with_mime_type (mime_type, error);
-        if (!loader)
-            return NULL;
-    }
-    else
-        loader = gdk_pixbuf_loader_new ();
-    if (!gdk_pixbuf_loader_write (loader, buffer, length, error))
-    {
-        g_object_unref (loader);
-        return NULL;
-    }
-    if (!gdk_pixbuf_loader_close (loader, error))
-    {
-        g_object_unref (loader);
-        return NULL;
-    }
-
-    pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
-    g_object_ref (pixbuf);
-    g_object_unref (loader);
-    return pixbuf;
-}
-
 /**
  * katze_tree_view_get_selected_iter:
  * @treeview: a #GtkTreeView
@@ -1138,29 +1087,6 @@ katze_skip_whitespace (const gchar* str)
     return str;
 }
 
-/**
- * katze_object_has_property:
- * @object: a #GObject
- * @property: the name of the property
- *
- * Determine if @object has a property with the specified name.
- *
- * Return value: a boolean
- *
- * Since: 0.1.2
- **/
-gboolean
-katze_object_has_property (gpointer     object,
-                           const gchar* property)
-{
-    GObjectClass* class;
-
-    g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
-
-    class = G_OBJECT_GET_CLASS (object);
-    return g_object_class_find_property (class, property) != NULL;
-}
-
 /**
  * katze_object_get_boolean:
  * @object: a #GObject
@@ -1205,28 +1131,6 @@ katze_object_get_int (gpointer     object,
     return value;
 }
 
-/**
- * katze_object_get_float:
- * @object: a #GObject
- * @property: the name of the property to get
- *
- * Retrieve the float value of the specified property.
- *
- * Return value: a float
- **/
-gfloat
-katze_object_get_float (gpointer     object,
-                        const gchar* property)
-{
-    gfloat value = -1.0f;
-
-    g_return_val_if_fail (G_IS_OBJECT (object), -1.0f);
-    /* FIXME: Check value type */
-
-    g_object_get (object, property, &value, NULL);
-    return value;
-}
-
 /**
  * katze_object_get_enum:
  * @object: a #GObject
