@@ -508,8 +508,7 @@ midori_app_command_received (MidoriApp*   app,
     {
         MidoriBrowser* browser = midori_app_create_browser (app);
         midori_app_add_browser (app, browser);
-        /* FIXME: Should open the homepage according to settings */
-        midori_browser_add_uri (browser, "");
+        midori_browser_add_uri (browser, "about:home");
         midori_browser_activate_action (browser, "Location");
         gtk_widget_show (GTK_WIDGET (browser));
         midori_app_raise_window (GTK_WINDOW (browser), screen);
@@ -572,15 +571,11 @@ midori_app_command_received (MidoriApp*   app,
     }
     else if (g_str_equal (command, "command"))
     {
-        guint i = 0;
-
         if (!uris || !app->browser)
             return FALSE;
-        while (uris[i] != NULL)
-        {
+        gint i;
+        for (i = 0; uris && uris[i]; i++)
             midori_browser_activate_action (app->browser, uris[i]);
-            i++;
-        }
         return TRUE;
     }
 
@@ -793,7 +788,7 @@ midori_app_init (MidoriApp* app)
     app->search_engines = NULL;
     app->history = NULL;
     app->speeddial = NULL;
-    app->extensions = NULL;
+    app->extensions = katze_array_new (KATZE_TYPE_ARRAY);
     app->browsers = katze_array_new (MIDORI_TYPE_BROWSER);
 
     app->instance = MidoriAppInstanceNull;
@@ -935,6 +930,25 @@ midori_app_new (const gchar* name)
     return g_object_new (MIDORI_TYPE_APP, "name", name, NULL);
 }
 
+/**
+ * midori_app_new_proxy:
+ * @app: a #MidoriApp, or %NULL
+ *
+ * Instantiates a proxy #MidoriApp that can be passed to untrusted code
+ * or for sensitive use cases. Properties can be freely changed.
+ *
+ * Return value: a new #MidoriApp
+ *
+ * Since: 0.5.0
+ **/
+MidoriApp*
+midori_app_new_proxy (MidoriApp* app)
+{
+    g_return_val_if_fail (MIDORI_IS_APP (app) || !app, NULL);
+
+    return midori_app_new (NULL);
+}
+
 static gboolean instance_is_not_running = FALSE;
 static gboolean instance_is_running = FALSE;
 
@@ -1072,7 +1086,7 @@ midori_app_instance_send_uris (MidoriApp* app,
         int i = 0;
         while (uris[i] != NULL)
         {
-            gchar* new_uri = sokoke_prepare_uri (uris[i]);
+            gchar* new_uri = sokoke_magic_uri (uris[i], TRUE, TRUE);
             gchar* escaped_uri = g_uri_escape_string (new_uri, NULL, FALSE);
             g_free (new_uri);
             katze_assign (uris[i], escaped_uri);
@@ -1118,20 +1132,15 @@ midori_app_send_command (MidoriApp* app,
     g_return_val_if_fail (MIDORI_IS_APP (app), FALSE);
     g_return_val_if_fail (command != NULL, FALSE);
 
-    if (midori_app_instance_is_running (app))
+    if (!midori_app_instance_is_running (app))
     {
         MidoriBrowser* browser = midori_browser_new ();
         int i;
         for (i=0; command && command[i]; i++)
-        {
-            gboolean action_known = (gtk_action_group_get_action (midori_browser_get_action_group (browser), command[i]) != NULL);
-            if (!action_known)
-                g_warning (_("Unexpected action '%s'."), command[i]);
-        }
+            midori_browser_assert_action (browser, command[i]);
         gtk_widget_destroy (GTK_WIDGET (browser));
-    }
-    else
         return midori_app_command_received (app, "command", command, NULL);
+    }
 
     #if HAVE_UNIQUE
     if (app->instance)
@@ -1440,7 +1449,20 @@ midori_app_setup (gint               *argc,
     g_object_unref (factory);
 
     if (!success)
-        g_error ("%s - %s", _("Midori"), error->message);
+        midori_error (error->message);
+}
+
+void
+midori_error (const gchar* format,
+              ...)
+{
+    g_printerr ("%s - ", _("Midori"));
+    va_list args;
+    va_start (args, format);
+    g_vfprintf (stderr, format, args);
+    va_end (args);
+    g_printerr ("\n");
+    exit (1);
 }
 
 gboolean
