@@ -580,10 +580,10 @@ midori_extension_load_from_file (const gchar* extension_path,
     typedef GObject* (*extension_init_func)(void);
     extension_init_func extension_init;
     static GHashTable* modules = NULL;
+    GObject* extension;
 
-    /* Ignore files which don't have the correct suffix */
-    if (!g_str_has_suffix (filename, G_MODULE_SUFFIX))
-        return NULL;
+    g_return_val_if_fail (extension_path != NULL, NULL);
+    g_return_val_if_fail (filename != NULL, NULL);
 
     if (strchr (filename, '/'))
     {
@@ -594,30 +594,38 @@ midori_extension_load_from_file (const gchar* extension_path,
     else
         fullname = g_build_filename (extension_path, filename, NULL);
 
+    /* Ignore files which don't have the correct suffix */
+    if (!g_str_has_suffix (fullname, G_MODULE_SUFFIX))
+    {
+        g_free (fullname);
+        return NULL;
+    }
+
     module = g_module_open (fullname, G_MODULE_BIND_LOCAL);
     g_free (fullname);
 
     /* GModule detects repeated loading but exposes no API to check it.
        Skip any modules that were loaded before. */
     if (modules == NULL)
-        modules = g_hash_table_new (g_direct_hash, g_direct_equal);
-    if (g_hash_table_lookup (modules, module))
-        return NULL;
-
-    g_hash_table_insert (modules, module, g_strdup (filename));
+        modules = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_object_unref);
+    if ((extension = g_hash_table_lookup (modules, module)))
+        return extension;
 
     if (module && g_module_symbol (module, "extension_init",
                                    (gpointer) &extension_init))
     {
         typedef void (*extension_test_func)(void);
         extension_test_func extension_test;
-        GObject* extension = extension_init ();
-        if (test && extension && g_module_symbol (module, "extension_test", (gpointer) &extension_test))
-            extension_test ();
-        return extension;
+        if ((extension = extension_init ()))
+        {
+            if (test && g_module_symbol (module, "extension_test", (gpointer) &extension_test))
+                extension_test ();
+            g_object_set_data_full (G_OBJECT (extension), "filename", g_strdup (filename), g_free);
+            g_hash_table_insert (modules, module, extension);
+        }
     }
 
-    return NULL;
+    return extension;
 }
 
 void
@@ -639,25 +647,34 @@ midori_extension_activate_gracefully (MidoriApp*   app,
         g_warning ("%s", g_module_error ());
         katze_array_add_item (extensions, extension);
         g_object_unref (extensions);
-    }
-
-    if (extension != NULL)
         g_object_unref (extension);
+    }
 }
 
 static void
-midori_load_extension (MidoriApp*       app,
-                       MidoriExtension* extension,
-                       const gchar*     filename)
+midori_extension_add_to_list (MidoriApp*       app,
+                              MidoriExtension* extension,
+                              const gchar*     filename)
 {
+    g_return_if_fail (MIDORI_IS_APP (app));
+    g_return_if_fail (filename != NULL);
     KatzeArray* extensions = katze_object_get_object (app, "extensions");
-    /* Signal that we want the extension to load and save */
-    g_object_set_data_full (G_OBJECT (extension), "filename",
-                            g_strdup (filename), g_free);
-    if (midori_extension_is_prepared (extension))
-        midori_extension_get_config_dir (extension);
+    g_return_if_fail (KATZE_IS_ARRAY (extensions));
+    if (katze_array_get_item_index (extensions, extension) >= 0)
+        return;
+
     katze_array_add_item (extensions, extension);
     g_object_unref (extensions);
+
+    if (midori_paths_is_readonly ())
+        return;
+
+    /* Signal that we want the extension to load and save */
+    if (midori_extension_is_prepared (extension))
+    {
+        g_warn_if_fail (extension->priv->config_dir == NULL);
+        extension->priv->config_dir = midori_paths_get_extension_config_dir (filename);
+    }
 }
 
 void
@@ -669,28 +686,38 @@ midori_extension_activate (GObject*     extension,
     if (MIDORI_IS_EXTENSION (extension))
     {
         if (filename != NULL)
-            midori_load_extension (app, MIDORI_EXTENSION (extension), filename);
+            midori_extension_add_to_list (app, MIDORI_EXTENSION (extension), filename);
         if (activate)
             g_signal_emit_by_name (extension, "activate", app);
     }
     else if (KATZE_IS_ARRAY (extension))
     {
+        gboolean success = FALSE;
         MidoriExtension* extension_item;
         KATZE_ARRAY_FOREACH_ITEM (extension_item, KATZE_ARRAY (extension))
             if (MIDORI_IS_EXTENSION (extension_item))
             {
-                gchar* key;
-
-                if (filename != NULL)
-                    midori_load_extension (app, extension_item, filename);
-                if (activate)
+                gchar* key = extension_item->priv->key;
+                g_return_if_fail (key != NULL);
+                if (filename != NULL && strchr (filename, '/'))
                 {
-                    key = katze_object_get_string (extension_item, "key");
-                    if (key && filename && strstr (filename, key))
-                        g_signal_emit_by_name (extension_item, "activate", app);
-                    g_free (key);
+                    gchar* clean = g_strndup (filename, strchr (filename, '/') - filename);
+                    g_object_set_data_full (G_OBJECT (extension_item), "filename", clean, g_free);
+                    midori_extension_add_to_list (app, extension_item, clean);
+                }
+                else if (filename != NULL)
+                {
+                    midori_extension_add_to_list (app, extension_item, filename);
+                    g_object_set_data_full (G_OBJECT (extension_item), "filename", g_strdup (filename), g_free);
+                }
+                if (activate && filename && strstr (filename, key))
+                {
+                    g_signal_emit_by_name (extension_item, "activate", app);
+                    success = TRUE;
                 }
             }
+        /* Passed a multi extension w/o key or non-existing key */
+        g_warn_if_fail (!activate || success);
     }
 }
 
@@ -827,15 +854,6 @@ midori_extension_get_config_dir (MidoriExtension* extension)
 
     g_return_val_if_fail (midori_extension_is_prepared (extension), NULL);
 
-    if (!extension->priv->config_dir)
-    {
-        gchar* filename = g_object_get_data (G_OBJECT (extension), "filename");
-        if (filename != NULL)
-            extension->priv->config_dir = midori_paths_get_extension_config_dir (filename);
-        else
-            extension->priv->config_dir = NULL;
-    }
-
     return extension->priv->config_dir;
 }
 
