@@ -45,6 +45,8 @@ midori_web_app_new (const gchar* config,
                     gint         inactivity_reset,
                     const gchar* block_uris)
 {
+    guint i;
+
     midori_paths_init (MIDORI_RUNTIME_MODE_APP, config);
 #ifndef HAVE_WEBKIT2
     g_object_set_data (G_OBJECT (webkit_get_default_session ()), "pass-through-console", (void*)1);
@@ -79,27 +81,27 @@ midori_web_app_new (const gchar* config,
 
     if (webapp != NULL)
     {
-        gchar* tmp_uri = sokoke_prepare_uri (webapp);
+        gchar* tmp_uri = sokoke_magic_uri (webapp, FALSE, TRUE);
         g_object_set (settings, "homepage", tmp_uri, NULL);
         midori_browser_add_uri (browser, tmp_uri);
         g_free (tmp_uri);
     }
-    else if (open_uris == NULL)
+
+    for (i = 0; open_uris && open_uris[i]; i++)
+    {
+        gchar* new_uri = sokoke_magic_uri (open_uris[i], FALSE, TRUE);
+        midori_browser_add_uri (browser, new_uri);
+        g_free (new_uri);
+    }
+    if (midori_browser_get_n_pages (browser) == 0)
         midori_browser_add_uri (browser, "about:blank");
     gtk_widget_show (GTK_WIDGET (browser));
 
-    guint i;
-    if (open_uris != NULL)
-        for (i = 0; open_uris[i] != NULL; i++)
-        {
-            gchar* new_uri = sokoke_prepare_uri (open_uris[i]);
-            midori_browser_add_uri (browser, new_uri);
-            g_free (new_uri);
-        }
-
-    if (execute_commands != NULL)
-        for (i = 0; execute_commands[i] != NULL; i++)
-            midori_browser_activate_action (browser, execute_commands[i]);
+    for (i = 0; execute_commands && execute_commands[i]; i++)
+    {
+        midori_browser_assert_action (browser, execute_commands[i]);
+        midori_browser_activate_action (browser, execute_commands[i]);
+    }
     return browser;
 }
 
@@ -121,7 +123,7 @@ midori_trash_remove_item_cb (KatzeArray* trash,
     gchar* config_file = midori_paths_get_config_filename_for_writing ("tabtrash.xbel");
     GError* error = NULL;
     midori_trash_add_item_no_save_cb (trash, item);
-    if (!midori_array_to_file (trash, config_file, "xbel", &error))
+    if (!midori_array_to_file (trash, config_file, "xbel-tiny", &error))
     {
         /* i18n: Trash, or wastebin, containing closed tabs */
         g_warning (_("The trash couldn't be saved. %s"), error->message);
@@ -145,6 +147,8 @@ midori_private_app_new (const gchar* config,
                         gint         inactivity_reset,
                         const gchar* block_uris)
 {
+    guint i;
+
     midori_paths_init (MIDORI_RUNTIME_MODE_PRIVATE, config);
 #ifndef HAVE_WEBKIT2
     g_object_set_data (G_OBJECT (webkit_get_default_session ()), "pass-through-console", (void*)1);
@@ -206,27 +210,27 @@ midori_private_app_new (const gchar* config,
 
     if (webapp != NULL)
     {
-        gchar* tmp_uri = sokoke_prepare_uri (webapp);
+        gchar* tmp_uri = sokoke_magic_uri (webapp, FALSE, TRUE);
         g_object_set (settings, "homepage", tmp_uri, NULL);
         midori_browser_add_uri (browser, tmp_uri);
         g_free (tmp_uri);
     }
-    else if (open_uris == NULL)
+
+    for (i = 0; open_uris && open_uris[i]; i++)
+    {
+        gchar* new_uri = sokoke_magic_uri (open_uris[i], FALSE, TRUE);
+        midori_browser_add_uri (browser, new_uri);
+        g_free (new_uri);
+    }
+    if (midori_browser_get_n_pages (browser) == 0)
         midori_browser_add_uri (browser, "about:private");
     gtk_widget_show (GTK_WIDGET (browser));
 
-    guint i;
-    if (open_uris != NULL)
-        for (i = 0; open_uris[i] != NULL; i++)
-        {
-            gchar* new_uri = sokoke_prepare_uri (open_uris[i]);
-            midori_browser_add_uri (browser, new_uri);
-            g_free (new_uri);
-        }
-
-    if (execute_commands != NULL)
-        for (i = 0; execute_commands[i] != NULL; i++)
-            midori_browser_activate_action (browser, execute_commands[i]);
+    for (i = 0; execute_commands && execute_commands[i]; i++)
+    {
+        midori_browser_assert_action (browser, execute_commands[i]);
+        midori_browser_activate_action (browser, execute_commands[i]);
+    }
     return browser;
 }
 
@@ -294,6 +298,27 @@ button_modify_preferences_clicked_cb (GtkWidget*         button,
         gtk_widget_destroy (dialog);
 }
 
+static void
+midori_frontend_crash_log_cb (GtkWidget* button,
+                              gchar*     crash_log)
+{
+    GError* error = NULL;
+    if (!sokoke_show_uri (gtk_widget_get_screen (button), crash_log, 0, &error))
+    {
+        sokoke_message_dialog (GTK_MESSAGE_ERROR,
+                               _("Could not run external program."),
+                               error->message, FALSE);
+        g_error_free (error);
+    }
+}
+
+static void
+midori_frontend_debugger_cb (GtkWidget* button,
+                             GtkDialog* dialog)
+{
+    gtk_dialog_response (dialog, GTK_RESPONSE_HELP);
+}
+
 static MidoriStartup
 midori_frontend_diagnostic_dialog (MidoriApp*         app,
                                    MidoriWebSettings* settings,
@@ -341,6 +366,29 @@ midori_frontend_diagnostic_dialog (MidoriApp*         app,
         _("Show last tabs without loading"), MIDORI_STARTUP_DELAYED_PAGES,
         _("Show last open tabs"), MIDORI_STARTUP_LAST_OPEN_PAGES,
         NULL);
+
+    gchar* crash_log = g_build_filename (midori_paths_get_runtime_dir (), "gdb.bt", NULL);
+    if (g_access (crash_log, F_OK) == 0)
+    {
+        GtkWidget* button = gtk_button_new_with_mnemonic (_("Show last crash _log"));
+        g_signal_connect_data (button, "clicked",
+            G_CALLBACK (midori_frontend_crash_log_cb), crash_log,
+            (GClosureNotify)g_free, 0);
+        gtk_widget_show (button);
+        gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 4);
+    }
+    else
+        g_free (crash_log);
+
+    gchar* gdb = g_find_program_in_path ("gdb");
+    if (gdb != NULL)
+    {
+        GtkWidget* button = gtk_button_new_with_mnemonic (_("Run in _debugger"));
+        g_signal_connect (button, "clicked",
+            G_CALLBACK (midori_frontend_debugger_cb), dialog);
+        gtk_widget_show (button);
+        gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 4);
+    }
     gtk_dialog_set_default_response (GTK_DIALOG (dialog),
         load_on_startup == MIDORI_STARTUP_HOMEPAGE
         ? MIDORI_STARTUP_BLANK_PAGE : load_on_startup);
@@ -364,6 +412,11 @@ midori_frontend_diagnostic_dialog (MidoriApp*         app,
     gtk_widget_destroy (dialog);
     if (response == GTK_RESPONSE_DELETE_EVENT)
         response = G_MAXINT;
+    else if (response == GTK_RESPONSE_HELP)
+    {
+        sokoke_spawn_gdb (gdb, FALSE);
+        response = G_MAXINT;
+    }
     else if (response == MIDORI_STARTUP_BLANK_PAGE)
         katze_array_clear (session);
     return response;
@@ -392,7 +445,7 @@ midori_normal_app_new (const gchar* config,
         gboolean result = FALSE;
         GtkWidget* dialog;
 
-        if (execute_commands != NULL && midori_app_send_command (app, open_uris))
+        if (execute_commands != NULL && midori_app_send_command (app, execute_commands))
             return NULL;
         if (open_uris != NULL && midori_app_instance_send_uris (app, open_uris))
             return NULL;
@@ -445,7 +498,7 @@ midori_normal_app_new (const gchar* config,
     {
         katze_assign (config_file, midori_paths_get_config_filename_for_reading ("session.xbel"));
         error = NULL;
-        if (!midori_array_from_file (session, config_file, "xbel", &error))
+        if (!midori_array_from_file (session, config_file, "xbel-tiny", &error))
         {
             if (error->code != G_FILE_ERROR_NOENT)
                 g_string_append_printf (error_messages,
@@ -461,7 +514,7 @@ midori_normal_app_new (const gchar* config,
         G_CALLBACK (midori_trash_remove_item_cb), NULL);
     katze_assign (config_file, g_build_filename (config, "tabtrash.xbel", NULL));
     error = NULL;
-    if (!midori_array_from_file (trash, config_file, "xbel", &error))
+    if (!midori_array_from_file (trash, config_file, "xbel-tiny", &error))
     {
         if (error->code != G_FILE_ERROR_NOENT)
             g_string_append_printf (error_messages,
