@@ -298,7 +298,8 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     {
     PAGE_NEW (GTK_STOCK_HOME, _("Startup"));
     FRAME_NEW (NULL);
-    label = katze_property_label (settings, "load-on-startup");
+    label = gtk_label_new (_("When Midori starts:"));
+    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
     INDENTED_ADD (label);
     button = katze_property_proxy (settings, "load-on-startup", NULL);
     SPANNED_ADD (button);
@@ -307,7 +308,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     INDENTED_ADD (label);
     entry = katze_property_proxy (settings, "homepage", "address");
     SPANNED_ADD (entry);
-    if (parent && katze_object_has_property (parent, "uri"))
+    if (parent && MIDORI_IS_BROWSER (parent))
     {
         #if 0
         button = gtk_button_new_with_mnemonic (_("Use _current page"));
@@ -377,12 +378,26 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
         else
             g_free (enchant_path);
     }
-    button = katze_property_proxy (settings, "enable-scripts", NULL);
+    button = katze_property_proxy (settings, "enable-javascript", NULL);
     gtk_button_set_label (GTK_BUTTON (button), _("Enable scripts"));
     INDENTED_ADD (button);
     #if WEBKIT_CHECK_VERSION (1, 3, 8)
     button = katze_property_proxy (settings, "enable-webgl", NULL);
     gtk_button_set_label (GTK_BUTTON (button), _("Enable WebGL support"));
+    #ifndef HAVE_WEBKIT2
+    if (parent && MIDORI_IS_BROWSER (parent))
+    {
+        GtkWidget* tab = midori_browser_get_current_tab (MIDORI_BROWSER (parent));
+        WebKitWebView* web_view = midori_tab_get_web_view (MIDORI_TAB (tab));
+        WebKitWebFrame* web_frame = webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW (web_view));
+        JSContextRef js_context = webkit_web_frame_get_global_context (web_frame);
+        gchar* supports_web_gl = sokoke_js_script_eval (js_context,
+            "!!window.WebGLRenderingContext", NULL);
+        if (g_strcmp0 (supports_web_gl, "true"))
+            gtk_widget_set_sensitive (button, FALSE);
+        g_free (supports_web_gl);
+    }
+    #endif
     #else
     button = katze_property_proxy (settings, "enable-plugins", NULL);
     gtk_button_set_label (GTK_BUTTON (button), _("Enable Netscape plugins"));
@@ -396,15 +411,6 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     gtk_button_set_label (GTK_BUTTON (button), _("Allow scripts to open popups"));
     gtk_widget_set_tooltip_text (button, _("Whether scripts are allowed to open popup windows automatically"));
     SPANNED_ADD (button);
-    button = katze_property_proxy (settings, "middle-click-opens-selection", NULL);
-    gtk_button_set_label (GTK_BUTTON (button), _("Middle click opens Selection"));
-    gtk_widget_set_tooltip_text (button, _("Load an address from the selection via middle click"));
-    INDENTED_ADD (button);
-    #ifndef G_OS_WIN32
-    button = katze_property_proxy (settings, "flash-window-on-new-bg-tabs", NULL);
-    gtk_button_set_label (GTK_BUTTON (button), _("Flash window on background tabs"));
-    SPANNED_ADD (button);
-    #endif
 
     FRAME_NEW (NULL);
     button = gtk_label_new (_("Preferred languages"));
@@ -421,18 +427,19 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
 
     /* Page "Interface" */
     PAGE_NEW (GTK_STOCK_CONVERT, _("Browsing"));
-    if (!g_getenv ("DESKTOP_SESSION"))
-    {
         FRAME_NEW (NULL);
-        INDENTED_ADD (katze_property_label (settings, "toolbar-style"));
+        INDENTED_ADD (gtk_label_new (_("Toolbar Style:")));
         button = katze_property_proxy (settings, "toolbar-style", NULL);
         SPANNED_ADD (button);
-    }
     FRAME_NEW (NULL);
-    label = katze_property_label (settings, "open-new-pages-in");
+    label = gtk_label_new (_("Open new pages in:"));
     INDENTED_ADD (label);
     button = katze_property_proxy (settings, "open-new-pages-in", NULL);
     SPANNED_ADD (button);
+    label = gtk_label_new (_("New tab behavior:"));
+    INDENTED_ADD (label);
+    button = katze_property_proxy (settings, "new-tab-type", "custom-tabhome");
+    SPANNED_ADD (button);
     button = katze_property_proxy (settings, "close-buttons-on-tabs", NULL);
     gtk_button_set_label (GTK_BUTTON (button), _("Close Buttons on Tabs"));
     INDENTED_ADD (button);
@@ -485,6 +492,27 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     g_signal_connect (settings, "notify::proxy-type",
         G_CALLBACK (midori_preferences_notify_proxy_type_cb), entry);
     midori_preferences_notify_proxy_type_cb (settings, NULL, entry);
+    #if GLIB_CHECK_VERSION (2, 26, 0)
+    INDENTED_ADD (gtk_event_box_new ());
+    label = gtk_label_new (NULL);
+    GString* proxy_types = g_string_new (NULL);
+    g_string_append (proxy_types, "<span size=\"smaller\">");
+    g_string_append (proxy_types, _("Supported proxy types:"));
+    g_string_append_c (proxy_types, ' ');
+    /* http is not implemented as a GIO extension */
+    g_string_append (proxy_types, "http:// https://");
+    GIOExtensionPoint* proxy_extension = g_io_extension_point_lookup ("gio-proxy");
+    GList* proxies = g_io_extension_point_get_extensions (proxy_extension);
+    for (; proxies; proxies = g_list_next (proxies))
+        g_string_append_printf (proxy_types, " %s://", g_io_extension_get_name (proxies->data));
+    g_string_append (proxy_types, "</span>");
+    gtk_label_set_markup (GTK_LABEL (label), proxy_types->str);
+    g_string_free (proxy_types, TRUE);
+    SPANNED_ADD (label);
+    g_signal_connect (settings, "notify::proxy-type",
+        G_CALLBACK (midori_preferences_notify_proxy_type_cb), label);
+    midori_preferences_notify_proxy_type_cb (settings, NULL, label);
+    #endif
     #if WEBKIT_CHECK_VERSION (1, 3, 11)
 #ifndef HAVE_WEBKIT2
     if (soup_session_get_feature (webkit_get_default_session (), SOUP_TYPE_CACHE))
@@ -501,7 +529,9 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     }
 #endif
     #endif
-    label = katze_property_label (settings, "identify-as");
+    /* i18n: This refers to an application, not the 'user agent' string */
+    label = gtk_label_new (_("Identify as"));
+    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
     INDENTED_ADD (label);
     button = katze_property_proxy (settings, "identify-as", "custom-user-agent");
     SPANNED_ADD (button);
