@@ -32,8 +32,12 @@ midori_soup_session_set_proxy_uri (SoupSession* session,
     SoupURI* proxy_uri;
 
     /* soup_uri_new expects a non-NULL string with a protocol */
-    if (midori_uri_is_http (uri))
+    gchar* scheme = uri ? g_uri_parse_scheme (uri): NULL;
+    if (scheme)
+    {
         proxy_uri = soup_uri_new (uri);
+        g_free (scheme);
+    }
     else if (uri && *uri)
     {
         gchar* fixed_uri = g_strconcat ("http://", uri, NULL);
@@ -367,7 +371,7 @@ midori_browser_action_last_session_activate_cb (GtkAction*     action,
     KatzeArray* old_session = katze_array_new (KATZE_TYPE_ITEM);
     gchar* config_file = midori_paths_get_config_filename_for_reading ("session.old.xbel");
     GError* error = NULL;
-    if (midori_array_from_file (old_session, config_file, "xbel", &error))
+    if (midori_array_from_file (old_session, config_file, "xbel-tiny", &error))
     {
         KatzeItem* item;
         KATZE_ARRAY_FOREACH_ITEM (item, old_session)
@@ -375,8 +379,8 @@ midori_browser_action_last_session_activate_cb (GtkAction*     action,
     }
     else
     {
-        g_warning (_("The session couldn't be loaded: %s\n"), error->message);
-        /* FIXME: Show a graphical dialog */
+        sokoke_message_dialog (GTK_MESSAGE_ERROR,
+            _("The session couldn't be loaded: %s\n"), error->message, FALSE);
         g_error_free (error);
     }
     g_free (config_file);
@@ -403,7 +407,7 @@ midori_session_save_timeout_cb (KatzeArray* session)
 {
     gchar* config_file = midori_paths_get_config_filename_for_writing ("session.xbel");
     GError* error = NULL;
-    if (!midori_array_to_file (session, config_file, "xbel", &error))
+    if (!midori_array_to_file (session, config_file, "xbel-tiny", &error))
     {
         g_warning (_("The session couldn't be saved. %s"), error->message);
         g_error_free (error);
@@ -482,9 +486,9 @@ midori_load_session (gpointer data)
     {
         item = katze_item_new ();
         if (load_on_startup == MIDORI_STARTUP_BLANK_PAGE)
-            katze_item_set_uri (item, "about:blank");
+            katze_item_set_uri (item, "about:new");
         else
-            item->uri = katze_object_get_string (settings, "homepage");
+            katze_item_set_uri (item, "about:home");
         katze_array_add_item (saved_session, item);
         g_object_unref (item);
     }
@@ -510,16 +514,12 @@ midori_load_session (gpointer data)
     if (midori_uri_is_blank (katze_item_get_uri (item)))
         midori_browser_activate_action (browser, "Location");
 
-    if (open_uris != NULL)
+    guint i = 0;
+    for (i = 0; open_uris && open_uris[i]; i++)
     {
-        guint i = 0;
-        while (open_uris[i])
-        {
-            gchar* uri = sokoke_prepare_uri (open_uris[i]);
-            midori_browser_add_uri (browser, uri);
-            g_free (uri);
-            i++;
-        }
+        gchar* uri = sokoke_magic_uri (open_uris[i], TRUE, TRUE);
+        midori_browser_add_uri (browser, uri);
+        g_free (uri);
     }
 
     g_object_unref (settings);
