@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2008 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2008-2013 Christian Dywan <christian@twotoasts.de>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -243,6 +243,11 @@ midori_view_list_plugins               (MidoriView*        view,
                                         GString*           markup,
                                         gboolean           html);
 
+void
+midori_view_set_colors                 (MidoriView*        view,
+                                        GdkColor*          fg_color,
+                                        GdkColor*          bg_color);
+
 G_END_DECLS
 
 #endif /* __MIDORI_VIEW_H__ */
