@@ -33,12 +33,19 @@
     #include <sys/sysctl.h>
 #endif
 
+#ifdef HAVE_WEBKIT2
+#define WEB_SETTINGS_STRING(x) "WebKitSettings::"x""
+#else
+#define WEB_SETTINGS_STRING(x) "WebKitWebSettings::"x""
+#endif
+
 struct _MidoriWebSettings
 {
     MidoriSettings parent_instance;
 
     MidoriToolbarStyle toolbar_style : 3;
     MidoriStartup load_on_startup : 2;
+    MidoriNewTabType new_tab_type: 3;
     MidoriPreferredEncoding preferred_encoding : 3;
     gint close_buttons_left;
     MidoriNewPage open_new_pages_in : 2;
@@ -76,6 +83,7 @@ enum
     PROP_TOOLBAR_STYLE,
 
     PROP_LOAD_ON_STARTUP,
+    PROP_NEW_TAB,
     PROP_PREFERRED_ENCODING,
 
     PROP_CLOSE_BUTTONS_LEFT,
@@ -114,6 +122,25 @@ midori_startup_get_type (void)
     return type;
 }
 
+GType
+midori_newtab_get_type (void)
+{
+    static GType type = 0;
+    if (!type)
+    {
+        static const GEnumValue values[] = {
+         { MIDORI_NEWTAB_BLANK_PAGE, "MIDORI_NEWTAB_BLANK_PAGE", N_("Show Blank Page") },
+         { MIDORI_NEWTAB_HOMEPAGE, "MIDORI_NEWTAB_HOMEPAGE", N_("Show Homepage") },
+         { MIDORI_NEWTAB_SEARCH, "MIDORI_NEWTAB_SEARCH", N_("Show default Search Engine") },
+         { MIDORI_NEWTAB_SPEED_DIAL, "MIDORI_NEWTAB_SPEED_DIAL", N_("Show Speed Dial") },
+         { MIDORI_NEWTAB_CUSTOM, "MIDORI_NEWTAB_CUSTOM", N_("Show custom page") },
+         { 0, NULL, NULL }
+        };
+        type = g_enum_register_static ("MidoriNewTabType", values);
+    };
+    return type;
+}
+
 GType
 midori_preferred_encoding_get_type (void)
 {
@@ -128,7 +155,7 @@ midori_preferred_encoding_get_type (void)
          { MIDORI_ENCODING_RUSSIAN, "MIDORI_ENCODING_RUSSIAN", N_("Russian (KOI8-R)") },
          { MIDORI_ENCODING_UNICODE, "MIDORI_ENCODING_UNICODE", N_("Unicode (UTF-8)") },
          { MIDORI_ENCODING_WESTERN, "MIDORI_ENCODING_WESTERN", N_("Western (ISO-8859-1)") },
-         { MIDORI_ENCODING_CUSTOM, "MIDORI_ENCODING_CUSTOM", N_("Custom...") },
+         { MIDORI_ENCODING_CUSTOM, "MIDORI_ENCODING_CUSTOM", N_("Custom…") },
          { 0, NULL, NULL }
         };
         type = g_enum_register_static ("MidoriPreferredEncoding", values);
@@ -204,7 +231,7 @@ midori_identity_get_type (void)
          { MIDORI_IDENT_IPHONE, "MIDORI_IDENT_IPHONE", N_("iPhone") },
          { MIDORI_IDENT_FIREFOX, "MIDORI_IDENT_FIREFOX", N_("Firefox") },
          { MIDORI_IDENT_EXPLORER, "MIDORI_IDENT_EXPLORER", N_("Internet Explorer") },
-         { MIDORI_IDENT_CUSTOM, "MIDORI_IDENT_CUSTOM", N_("Custom...") },
+         { MIDORI_IDENT_CUSTOM, "MIDORI_IDENT_CUSTOM", N_("Custom…") },
          { 0, NULL, NULL }
         };
         type = g_enum_register_static ("MidoriIdentity", values);
@@ -277,7 +304,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      PROP_TOOLBAR_STYLE,
                                      g_param_spec_enum (
                                      "toolbar-style",
-                                     _("Toolbar Style:"),
+                                     "Toolbar Style:",
                                      _("The style of the toolbar"),
                                      MIDORI_TYPE_TOOLBAR_STYLE,
                                      MIDORI_TOOLBAR_DEFAULT,
@@ -287,12 +314,22 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      PROP_LOAD_ON_STARTUP,
                                      g_param_spec_enum (
                                      "load-on-startup",
-                                     _("When Midori starts:"),
+                                     "When Midori starts:",
                                      "What to do when Midori starts",
                                      MIDORI_TYPE_STARTUP,
                                      MIDORI_STARTUP_LAST_OPEN_PAGES,
                                      flags));
 
+    g_object_class_install_property (gobject_class,
+                                     PROP_NEW_TAB,
+                                     g_param_spec_enum (
+                                     "new-tab-type",
+                                     "New tab behavior:",
+                                     "What to show in newly opened tabs",
+                                     MIDORI_TYPE_NEWTAB,
+                                     MIDORI_NEWTAB_SPEED_DIAL,
+                                     flags));
+
     g_object_class_install_property (gobject_class,
                                      PROP_PREFERRED_ENCODING,
                                      g_param_spec_enum (
@@ -324,7 +361,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      PROP_OPEN_NEW_PAGES_IN,
                                      g_param_spec_enum (
                                      "open-new-pages-in",
-                                     _("Open new pages in:"),
+                                     "Open new pages in:",
                                      "Where to open new pages",
                                      MIDORI_TYPE_NEW_PAGE,
                                      MIDORI_NEW_PAGE_TAB,
@@ -386,9 +423,8 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      PROP_IDENTIFY_AS,
                                      g_param_spec_enum (
                                      "identify-as",
-        /* i18n: This refers to an application, not the 'user agent' string */
-                                     _("Identify as"),
-                                     _("What to identify as to web pages"),
+                                     "Identify as",
+                                     "What to identify as to web pages",
                                      MIDORI_TYPE_IDENTITY,
                                      MIDORI_IDENT_MIDORI,
                                      flags));
@@ -708,6 +744,27 @@ midori_web_settings_get_system_name (gchar** architecture,
     #endif
 }
 
+static const gchar*
+get_uri_for_new_tab (MidoriWebSettings* web_settings,
+                     MidoriNewTabType   new_tab_type)
+{
+    switch (new_tab_type)
+    {
+        case MIDORI_NEWTAB_BLANK_PAGE:
+            return "about:blank";
+        case MIDORI_NEWTAB_HOMEPAGE:
+            return "about:home";
+        case MIDORI_NEWTAB_SEARCH:
+            return "about:search";
+        case MIDORI_NEWTAB_CUSTOM:
+            return midori_settings_get_tabhome (MIDORI_SETTINGS (web_settings));
+        case MIDORI_NEWTAB_SPEED_DIAL:
+            return "about:dial";
+        default:
+            g_assert_not_reached ();
+    }
+}
+
 static gchar*
 generate_ident_string (MidoriWebSettings* web_settings,
                        MidoriIdentity     identify_as)
@@ -938,7 +995,7 @@ midori_web_settings_set_property (GObject*      object,
 
     case PROP_ENABLE_PLUGINS:
         g_object_set (web_settings,
-            "WebKitWebSettings::enable-plugins", g_value_get_boolean (value),
+           WEB_SETTINGS_STRING ("enable-plugins"), g_value_get_boolean (value),
         #if WEBKIT_CHECK_VERSION (1, 1, 22)
             "enable-java-applet", g_value_get_boolean (value),
         #endif
@@ -946,7 +1003,7 @@ midori_web_settings_set_property (GObject*      object,
         break;
     #if WEBKIT_CHECK_VERSION (1, 1, 18)
     case PROP_ENABLE_PAGE_CACHE:
-        g_object_set (web_settings, "WebKitWebSettings::enable-page-cache",
+        g_object_set (web_settings, WEB_SETTINGS_STRING ("enable-page-cache"),
                       g_value_get_boolean (value), NULL);
         break;
     #endif
@@ -966,9 +1023,14 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_USER_AGENT:
         if (web_settings->identify_as == MIDORI_IDENT_CUSTOM)
             katze_assign (web_settings->ident_string, g_value_dup_string (value));
-        g_object_set (web_settings, "WebKitWebSettings::user-agent",
+        g_object_set (web_settings, WEB_SETTINGS_STRING ("user-agent"),
                                     web_settings->ident_string, NULL);
         break;
+    case PROP_NEW_TAB:
+        web_settings->new_tab_type = g_value_get_enum (value);
+        const gchar* tabhome = get_uri_for_new_tab (web_settings, web_settings->new_tab_type);
+        midori_settings_set_tabhome (MIDORI_SETTINGS (web_settings), tabhome);
+        break;
     case PROP_PREFERRED_LANGUAGES:
         katze_assign (web_settings->http_accept_language, g_value_dup_string (value));
         g_object_set (web_settings, "spell-checking-languages",
@@ -1003,7 +1065,7 @@ midori_web_settings_set_property (GObject*      object,
             midori_web_settings_remove_style (web_settings, "enforce-font-family");
         break;
     case PROP_ENABLE_FULLSCREEN:
-        g_object_set (web_settings, "WebKitWebSettings::enable-fullscreen",
+        g_object_set (web_settings, WEB_SETTINGS_STRING ("enable-fullscreen"),
                       g_value_get_boolean (value), NULL);
         break;
     case PROP_USER_STYLESHEET_URI:
@@ -1098,12 +1160,12 @@ midori_web_settings_get_property (GObject*    object,
 
     case PROP_ENABLE_PLUGINS:
         g_value_set_boolean (value, katze_object_get_boolean (web_settings,
-                             "WebKitWebSettings::enable-plugins"));
+                             WEB_SETTINGS_STRING ("enable-plugins")));
         break;
     #if WEBKIT_CHECK_VERSION (1, 1, 18)
     case PROP_ENABLE_PAGE_CACHE:
         g_value_set_boolean (value, katze_object_get_boolean (web_settings,
-                             "WebKitWebSettings::enable-page-cache"));
+                             WEB_SETTINGS_STRING ("enable-page-cache")));
         break;
     #endif
 
@@ -1121,6 +1183,9 @@ midori_web_settings_get_property (GObject*    object,
         }
         g_value_set_string (value, web_settings->ident_string);
         break;
+    case PROP_NEW_TAB:
+        g_value_set_enum (value, web_settings->new_tab_type);
+        break;
     case PROP_PREFERRED_LANGUAGES:
         g_value_set_string (value, web_settings->http_accept_language);
         break;
@@ -1137,11 +1202,11 @@ midori_web_settings_get_property (GObject*    object,
         break;
     case PROP_ENABLE_FULLSCREEN:
         g_value_set_boolean (value, katze_object_get_boolean (web_settings,
-            "WebKitWebSettings::enable-fullscreen"));
+            WEB_SETTINGS_STRING ("enable-fullscreen")));
         break;
     case PROP_USER_STYLESHEET_URI:
         g_value_take_string (value, katze_object_get_string (web_settings,
-            "WebKitWebSettings::user-stylesheet-uri"));
+            WEB_SETTINGS_STRING ("user-stylesheet-uri")));
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1196,7 +1261,7 @@ midori_web_settings_process_stylesheets (MidoriWebSettings* settings,
 
     /* data: uri prefix from Source/WebCore/page/Page.cpp:700 in WebKit */
     encoded = g_strconcat ("data:text/css;charset=utf-8;base64,", css->str, NULL);
-    g_object_set (G_OBJECT (settings), "WebKitWebSettings::user-stylesheet-uri", encoded, NULL);
+    g_object_set (settings, WEB_SETTINGS_STRING ("user-stylesheet-uri"), encoded, NULL);
     g_free (encoded);
     g_string_free (css, TRUE);
 }
@@ -1487,32 +1552,22 @@ midori_settings_save_to_file (MidoriWebSettings* settings,
     {
         i = 0;
         while (_extensions[i])
-        {
-            g_key_file_set_boolean (key_file, "extensions", _extensions[i], TRUE);
-            i++;
-        }
+            g_key_file_set_boolean (key_file, "extensions", _extensions[i++], TRUE);
     }
     else if (extensions)
     {
         KATZE_ARRAY_FOREACH_ITEM (extension, extensions)
             if (midori_extension_is_active (extension))
             {
-                const gchar* filename = g_object_get_data (
-                    G_OBJECT (extension), "filename");
-
-                gchar* key;
-                gchar* term;
-
-                key = katze_object_get_string (extension, "key");
-                if (key && *key)
-                    term = g_strdup_printf ("%s/%s", filename, key);
-                else
-                    term = g_strdup (filename);
-
-                g_key_file_set_boolean (key_file, "extensions", term, TRUE);
-
+                const gchar* filename = g_object_get_data (G_OBJECT (extension), "filename");
+                g_return_val_if_fail (filename != NULL, FALSE);
+                if (filename && strchr (filename, '/'))
+                    g_warning ("%s: %s unexpected /", G_STRFUNC, filename);
+                gchar* key = katze_object_get_string (extension, "key");
+                gchar* subname = key ? g_strdup_printf ("%s/%s", filename, key) : g_strdup (filename);
+                g_key_file_set_boolean (key_file, "extensions", subname, TRUE);
                 g_free (key);
-                g_free (term);
+                g_free (subname);
             }
         g_object_unref (extensions);
     }
