@@ -394,6 +394,22 @@ sokoke_spawn_program (const gchar* command,
     return TRUE;
 }
 
+void
+sokoke_spawn_gdb (const gchar* gdb,
+                  gboolean     sync)
+{
+    gchar* args = midori_paths_get_command_line_str (FALSE);
+    const gchar* runtime_dir = midori_paths_get_runtime_dir ();
+    gchar* cmd = g_strdup_printf (
+        "--batch -ex 'set print thread-events off' -ex run "
+        "-ex 'set logging on %s/%s' -ex 'bt' --return-child-result "
+        "--args %s",
+        runtime_dir, "gdb.bt", args);
+    sokoke_spawn_program (gdb, TRUE, cmd, FALSE, sync);
+    g_free (cmd);
+    g_free (args);
+}
+
 void
 sokoke_spawn_app (const gchar* uri,
                   gboolean     private)
@@ -470,25 +486,6 @@ sokoke_external_uri (const gchar* uri)
     return info != NULL;
 }
 
-gchar*
-sokoke_prepare_uri (const gchar *uri)
-{
-    gchar* uri_ready;
-
-    if (g_str_has_prefix (uri, "javascript:"))
-        return g_strdup (uri);
-    else if (g_file_test (uri, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)
-         && !g_path_is_absolute (uri))
-    {
-        GFile* file = g_file_new_for_commandline_arg (uri);
-        uri_ready = g_file_get_uri (file);
-        g_object_unref (file);
-        return uri_ready;
-    }
-
-    return sokoke_magic_uri (uri);
-}
-
 /**
  * sokoke_magic_uri:
  * @uri: a string typed by a user
@@ -501,7 +498,9 @@ sokoke_prepare_uri (const gchar *uri)
  * Return value: a newly allocated URI, or %NULL
  **/
 gchar*
-sokoke_magic_uri (const gchar* uri)
+sokoke_magic_uri (const gchar* uri,
+                  gboolean     allow_search,
+                  gboolean     allow_relative)
 {
     gchar** parts;
     gchar* search;
@@ -511,6 +510,14 @@ sokoke_magic_uri (const gchar* uri)
     /* Add file:// if we have a local path */
     if (g_path_is_absolute (uri))
         return g_filename_to_uri (uri, NULL, NULL);
+    if (allow_relative
+     && g_file_test (uri, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
+    {
+        GFile* file = g_file_new_for_commandline_arg (uri);
+        gchar* uri_ready = g_file_get_uri (file);
+        g_object_unref (file);
+        return uri_ready;
+    }
     /* Parse geo URI geo:48.202778,16.368472;crs=wgs84;u=40 as a location */
     if (!strncmp (uri, "geo:", 4))
     {
@@ -565,6 +572,8 @@ sokoke_magic_uri (const gchar* uri)
         }
         g_strfreev (parts);
     }
+    if (!allow_search)
+        midori_error (_("Invalid URI"));
     return NULL;
 }
 
