@@ -55,6 +55,10 @@ sokoke_spawn_program                    (const gchar* command,
                                          gboolean        quote_argument,
                                          gboolean        sync);
 
+void
+sokoke_spawn_gdb                        (const gchar*    gdb,
+                                         gboolean        sync);
+
 void
 sokoke_spawn_app                        (const gchar*    uri,
                                          gboolean        inherit_config);
@@ -63,10 +67,9 @@ gboolean
 sokoke_external_uri                     (const gchar*    uri);
 
 gchar*
-sokoke_prepare_uri                      (const gchar*    uri);
-
-gchar*
-sokoke_magic_uri                        (const gchar*    uri);
+sokoke_magic_uri                        (const gchar*    uri,
+                                         gboolean        allow_search,
+                                         gboolean        allow_realtive);
 
 void
 sokoke_widget_set_visible               (GtkWidget*      widget,
